@@ -73,6 +73,11 @@ done
 
 if [[ $root == *Linked* ]]; then 
 Linked=Linked
+
+# Macros for transforming the bi-directional long link. Return values are 32-bit int indexes.
+# SET_UPPER and SET_LOWER do a masked assignment as described at
+# http://www-graphics.stanford.edu/~seander/bithacks.html#MaskedMerge
+
 echo -e \
 "#define SET_PREV( f64, p32 )       SET_UPPER( f64, p32 )\n"\
 "#define SET_NEXT( f64, n32 )       SET_LOWER( f64, n32 )\n"\
@@ -143,6 +148,14 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#define KEY_GENERIC_CAST (K)\n"\
 "#define KEY_GENERIC_ARRAY_CAST (K[])\n"\
 "#define KEY_GENERIC_BIG_ARRAY_CAST (K[][])\n"\
+"#define SUPPRESS_WARNINGS_KEY_UNCHECKED @SuppressWarnings(\"unchecked\")\n"\
+"#define SUPPRESS_WARNINGS_KEY_RAWTYPES @SuppressWarnings(\"rawtypes\")\n"\
+"#define SUPPRESS_WARNINGS_KEY_UNCHECKED_RAWTYPES @SuppressWarnings({\"unchecked\",\"rawtypes\"})\n"\
+"#if defined(Custom) && #keyclass(Object)\n"\
+"#define SUPPRESS_WARNINGS_CUSTOM_KEY_UNCHECKED @SuppressWarnings(\"unchecked\")\n"\
+"#else\n"\
+"#define SUPPRESS_WARNINGS_CUSTOM_KEY_UNCHECKED\n"\
+"#endif\n"\
 "#else\n"\
 "#define KEY_GENERIC_CLASS KEY_CLASS\n"\
 "#define KEY_GENERIC_TYPE KEY_TYPE\n"\
@@ -153,6 +166,10 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#define KEY_GENERIC_CAST\n"\
 "#define KEY_GENERIC_ARRAY_CAST\n"\
 "#define KEY_GENERIC_BIG_ARRAY_CAST\n"\
+"#define SUPPRESS_WARNINGS_KEY_UNCHECKED\n"\
+"#define SUPPRESS_WARNINGS_KEY_RAWTYPES\n"\
+"#define SUPPRESS_WARNINGS_KEY_UNCHECKED_RAWTYPES\n"\
+"#define SUPPRESS_WARNINGS_CUSTOM_KEY_UNCHECKED\n"\
 "#endif\n"\
 \
 "#if #valueclass(Object) || #valueclass(Reference)\n"\
@@ -162,6 +179,8 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#define VALUE_EXTENDS_GENERIC <? extends V>\n"\
 "#define VALUE_GENERIC_CAST (V)\n"\
 "#define VALUE_GENERIC_ARRAY_CAST (V[])\n"\
+"#define SUPPRESS_WARNINGS_VALUE_UNCHECKED @SuppressWarnings(\"unchecked\")\n"\
+"#define SUPPRESS_WARNINGS_VALUE_RAWTYPES @SuppressWarnings(\"rawtypes\")\n"\
 "#else\n"\
 "#define VALUE_GENERIC_CLASS VALUE_CLASS\n"\
 "#define VALUE_GENERIC_TYPE VALUE_TYPE\n"\
@@ -169,6 +188,8 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#define VALUE_EXTENDS_GENERIC\n"\
 "#define VALUE_GENERIC_CAST\n"\
 "#define VALUE_GENERIC_ARRAY_CAST\n"\
+"#define SUPPRESS_WARNINGS_VALUE_UNCHECKED\n"\
+"#define SUPPRESS_WARNINGS_VALUE_RAWTYPES\n"\
 "#endif\n"\
 \
 "#if #keyclass(Object) || #keyclass(Reference)\n"\
@@ -189,6 +210,14 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#endif\n"\
 "#endif\n"\
 \
+"#if #keyclass(Object) || #keyclass(Reference) || #valueclass(Object) || #valueclass(Reference)\n"\
+"#define SUPPRESS_WARNINGS_KEY_VALUE_UNCHECKED @SuppressWarnings(\"unchecked\")\n"\
+"#define SUPPRESS_WARNINGS_KEY_VALUE_RAWTYPES @SuppressWarnings(\"rawtypes\")\n"\
+"#else\n"\
+"#define SUPPRESS_WARNINGS_KEY_VALUE_UNCHECKED\n"\
+"#define SUPPRESS_WARNINGS_KEY_VALUE_RAWTYPES\n"\
+"#endif\n"\
+\
 \
 "/* Value methods */\n"\
 \
@@ -310,6 +339,7 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#define OPEN_HASH_BIG_SET ${TYPE_CAP[$k]}${Linked}Open${Custom}HashBigSet\n\n"\
 "#define OPEN_DOUBLE_HASH_SET ${TYPE_CAP[$k]}${Linked}Open${Custom}DoubleHashSet\n\n"\
 "#define OPEN_HASH_MAP ${TYPE_CAP[$k]}2${TYPE_CAP[$v]}${Linked}Open${Custom}HashMap\n\n"\
+"#define OPEN_HASH_BIG_MAP ${TYPE_CAP[$k]}2${TYPE_CAP[$v]}${Linked}Open${Custom}HashBigMap\n\n"\
 "#define STRIPED_OPEN_HASH_MAP Striped${TYPE_CAP[$k]}2${TYPE_CAP[$v]}Open${Custom}HashMap\n\n"\
 "#define OPEN_DOUBLE_HASH_MAP ${TYPE_CAP[$k]}2${TYPE_CAP[$v]}${Linked}Open${Custom}DoubleHashMap\n\n"\
 "#define ARRAY_SET ${TYPE_CAP[$k]}ArraySet\n\n"\
@@ -436,24 +466,31 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 \
 \
 \
-"#ifdef Custom\n"\
-"#define KEY_EQUALS(x,y) ( strategy.equals( (x), " KEY_GENERIC_CAST "(y) ) )\n"\
-"#else\n"\
+"#define KEY_EQUALS_NOT_NULL_CAST(x,y) KEY_EQUALS_NOT_NULL(x,y)\n"\
+"#define KEY2INTHASH_CAST(x) KEY2INTHASH(x)\n"\
 "#if #keyclass(Object)\n"\
+"#ifdef Custom\n"\
+"#define KEY_EQUALS(x,y) ( strategy.equals( (x), (y) ) )\n"\
+"#define KEY_EQUALS_NOT_NULL(x,y) ( strategy.equals( (x), (y) ) )\n"\
+"#undef KEY_EQUALS_NOT_NULL_CAST\n"\
+"#define KEY_EQUALS_NOT_NULL_CAST(x,y) ( strategy.equals( " KEY_GENERIC_CAST "(x), (y) ) )\n"\
+"#else\n"\
 "#define KEY_EQUALS(x,y) ( (x) == null ? (y) == null : (x).equals(y) )\n"\
 "#define KEY_EQUALS_NOT_NULL(x,y) ( (x).equals(y) )\n"\
-"#else\n"\
-"#if #keyclass(Float)\n"\
+"#endif\n"\
+"#define KEY_IS_NULL(x) ( (x) == null )\n"\
+"#elif #keyclass(Float)\n"\
 "#define KEY_EQUALS(x,y) ( Float.floatToIntBits(x) == Float.floatToIntBits(y) )\n"\
 "#define KEY_EQUALS_NOT_NULL(x,y) ( Float.floatToIntBits(x) == Float.floatToIntBits(y) )\n"\
+"#define KEY_IS_NULL(x) ( Float.floatToIntBits(x) == 0 )\n"\
 "#elif #keyclass(Double)\n"\
 "#define KEY_EQUALS(x,y) ( Double.doubleToLongBits(x) == Double.doubleToLongBits(y) )\n"\
 "#define KEY_EQUALS_NOT_NULL(x,y) ( Double.doubleToLongBits(x) == Double.doubleToLongBits(y) )\n"\
+"#define KEY_IS_NULL(x) ( Double.doubleToLongBits(x) == 0 )\n"\
 "#else\n"\
 "#define KEY_EQUALS(x,y) ( (x) == (y) )\n"\
 "#define KEY_EQUALS_NOT_NULL(x,y) ( (x) == (y) )\n"\
-"#endif\n"\
-"#endif\n"\
+"#define KEY_IS_NULL(x) ( (x) == KEY_NULL )\n"\
 "#endif\n\n"\
 \
 "#if #valueclass(Object)\n"\
@@ -477,18 +514,21 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 \
 "#if #keyclass(Object)\n"\
 "#ifdef Custom\n"\
-"#define KEY2JAVAHASH(x) ( strategy.hashCode(" KEY_GENERIC_CAST "(x)) )\n"\
-"#define KEY2INTHASH(x) ( it.unimi.dsi.fastutil.HashCommon.murmurHash3( strategy.hashCode(" KEY_GENERIC_CAST "(x)) ^ mask ) )\n"\
-"#define KEY2LONGHASH(x) ( it.unimi.dsi.fastutil.HashCommon.murmurHash3( (long)( strategy.hashCode(" KEY_GENERIC_CAST "(x)) ^ mask ) ) )\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) ( strategy.hashCode(x) )\n"\
+"#define KEY2INTHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( strategy.hashCode(x) ) )\n"\
+"#undef KEY2INTHASH_CAST\n"\
+"#define KEY2INTHASH_CAST(x) ( it.unimi.dsi.fastutil.HashCommon.mix( strategy.hashCode( " KEY_GENERIC_CAST " x) ) )\n"\
+"#define KEY2LONGHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( (long)( strategy.hashCode(x)) ) ) )\n"\
 "#else\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) ( (x).hashCode() )\n"\
 "#define KEY2JAVAHASH(x) ( (x) == null ? 0 : (x).hashCode() )\n"\
-"#define KEY2INTHASH(x) ( (x) == null ? 0x87fcd5c : it.unimi.dsi.fastutil.HashCommon.murmurHash3( (x).hashCode() ^ mask ) )\n"\
-"#define KEY2LONGHASH(x) ( (x) == null ? 0x810879608e4259ccL : it.unimi.dsi.fastutil.HashCommon.murmurHash3( (long)( (x).hashCode() ^ mask ) ) )\n"\
+"#define KEY2INTHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( (x).hashCode() ) )\n"\
+"#define KEY2LONGHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( (long)( (x).hashCode() ) ) )\n"\
 "#endif\n"\
 "#else\n"\
-"#define KEY2JAVAHASH(x) ( (x) == null ? 0 : System.identityHashCode(x) )\n"\
-"#define KEY2INTHASH(x) ( (x) == null ? 0x87fcd5c : it.unimi.dsi.fastutil.HashCommon.murmurHash3( System.identityHashCode(x) ^ mask ) )\n"\
-"#define KEY2LONGHASH(x) ( (x) == null ? 0x810879608e4259ccL : it.unimi.dsi.fastutil.HashCommon.murmurHash3( (long)( System.identityHashCode(x) ^ mask ) ) )\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) ( System.identityHashCode(x) )\n"\
+"#define KEY2INTHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( System.identityHashCode(x) ) )\n"\
+"#define KEY2LONGHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( (long)( System.identityHashCode(x) ) ) )\n"\
 "#endif\n"\
 \
 "#define KEY_CMP(x,y) ( ((Comparable<KEY_GENERIC_CLASS>)(x)).compareTo(y) )\n"\
@@ -514,11 +554,15 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#if #keyclass(Boolean)\n"\
 "#define KEY_CMP_EQ(x,y) ( (x) == (y) )\n"\
 "#define KEY_NULL (false)\n"\
-"#define KEY_CMP(x,y) ( !(x) && (y) ? -1 : ( (x) == (y) ? 0 : 1 ) )\n"\
+"#define KEY_CMP(x,y) ( KEY_CLASS.compare((x),(y)) )\n"\
 "#define KEY_LESS(x,y) ( !(x) && (y) )\n"\
 "#define KEY_LESSEQ(x,y) ( !(x) || (y) )\n"\
 "#else\n"\
+"#if #keyclass(Byte) || #keyclass(Short) || #keyclass(Character)\n"\
 "#define KEY_NULL ((KEY_TYPE)0)\n"\
+"#else\n"\
+"#define KEY_NULL (0)\n"\
+"#endif\n"\
 "#if #keyclass(Float) || #keyclass(Double)\n"\
 "#define KEY_CMP_EQ(x,y) ( KEY_CLASS.compare((x),(y)) == 0 )\n"\
 "#define KEY_CMP(x,y) ( KEY_CLASS.compare((x),(y)) )\n"\
@@ -526,7 +570,7 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#define KEY_LESSEQ(x,y) ( KEY_CLASS.compare((x),(y)) <= 0 )\n"\
 "#else\n"\
 "#define KEY_CMP_EQ(x,y) ( (x) == (y) )\n"\
-"#define KEY_CMP(x,y) ( (x) < (y) ? -1 : ( (x) == (y) ? 0 : 1 ) )\n"\
+"#define KEY_CMP(x,y) ( KEY_CLASS.compare((x),(y)) )\n"\
 "#define KEY_LESS(x,y) ( (x) < (y) )\n"\
 "#define KEY_LESSEQ(x,y) ( (x) <= (y) )\n"\
 "#endif\n"\
@@ -542,36 +586,44 @@ $(if [[ "${CLASS[$v]}" != "" ]]; then\
 "#endif\n"\
 \
 "#ifdef Custom\n"\
-"#define KEY2JAVAHASH(x) ( strategy.hashCode(x) )\n"\
-"#define KEY2INTHASH(x) ( it.unimi.dsi.fastutil.HashCommon.murmurHash3( strategy.hashCode(x) ^ mask ) )\n"\
-"#define KEY2LONGHASH(x) ( it.unimi.dsi.fastutil.HashCommon.murmurHash3( (long)( strategy.hashCode(x) ^ mask ) ) )\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) ( strategy.hashCode(x) )\n"\
+"#define KEY2INTHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( strategy.hashCode(x) ) )\n"\
+"#define KEY2LONGHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( (long)( strategy.hashCode(x) ) ) )\n"\
 "#else\n"\
 \
 "#if #keyclass(Float)\n"\
-"#define KEY2JAVAHASH(x) it.unimi.dsi.fastutil.HashCommon.float2int(x)\n"\
-"#define KEY2INTHASH(x) it.unimi.dsi.fastutil.HashCommon.murmurHash3( it.unimi.dsi.fastutil.HashCommon.float2int(x) ^ mask )\n"\
-"#define KEY2LONGHASH(x) it.unimi.dsi.fastutil.HashCommon.murmurHash3( (long)( it.unimi.dsi.fastutil.HashCommon.float2int(x) ^ mask ) )\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) it.unimi.dsi.fastutil.HashCommon.float2int(x)\n"\
+"#define KEY2INTHASH(x) it.unimi.dsi.fastutil.HashCommon.mix( it.unimi.dsi.fastutil.HashCommon.float2int(x) )\n"\
+"#define KEY2LONGHASH(x) it.unimi.dsi.fastutil.HashCommon.mix( (long)( it.unimi.dsi.fastutil.HashCommon.float2int(x) ) )\n"\
+"#define INT(x) (x)\n"\
 "#elif #keyclass(Double)\n"\
-"#define KEY2JAVAHASH(x) it.unimi.dsi.fastutil.HashCommon.double2int(x)\n"\
-"#define KEY2INTHASH(x) (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3( Double.doubleToRawLongBits(x) ^ mask )\n"\
-"#define KEY2LONGHASH(x) it.unimi.dsi.fastutil.HashCommon.murmurHash3( Double.doubleToRawLongBits(x) ^ mask )\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) it.unimi.dsi.fastutil.HashCommon.double2int(x)\n"\
+"#define KEY2INTHASH(x) (int)it.unimi.dsi.fastutil.HashCommon.mix( Double.doubleToRawLongBits(x) )\n"\
+"#define KEY2LONGHASH(x) it.unimi.dsi.fastutil.HashCommon.mix( Double.doubleToRawLongBits(x) )\n"\
+"#define INT(x) (int)(x)\n"\
 "#elif #keyclass(Long)\n"\
-"#define KEY2JAVAHASH(x) it.unimi.dsi.fastutil.HashCommon.long2int(x)\n"\
-"#define KEY2INTHASH(x) (int)it.unimi.dsi.fastutil.HashCommon.murmurHash3( (x) ^ mask )\n"\
-"#define KEY2LONGHASH(x) it.unimi.dsi.fastutil.HashCommon.murmurHash3( (x) ^ mask )\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) it.unimi.dsi.fastutil.HashCommon.long2int(x)\n"\
+"#define KEY2INTHASH(x) (int)it.unimi.dsi.fastutil.HashCommon.mix( (x) )\n"\
+"#define KEY2LONGHASH(x) it.unimi.dsi.fastutil.HashCommon.mix( (x) )\n"\
+"#define INT(x) (int)(x)\n"\
 "#elif #keyclass(Boolean)\n"\
-"#define KEY2JAVAHASH(x) ((x) ? 1231 : 1237)\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) ((x) ? 1231 : 1237)\n"\
 "#define KEY2INTHASH(x) ((x) ? 0xfab5368 : 0xcba05e7b)\n"\
 "#define KEY2LONGHASH(x) ((x) ? 0x74a19fc8b6428188L : 0xbaeca2031a4fd9ecL)\n"\
 "#else\n"\
-"#define KEY2JAVAHASH(x) (x)\n"\
-"#define KEY2INTHASH(x) ( it.unimi.dsi.fastutil.HashCommon.murmurHash3( (x) ^ mask ) )\n"\
-"#define KEY2LONGHASH(x) ( it.unimi.dsi.fastutil.HashCommon.murmurHash3( (long)( (x) ^ mask ) ) )\n"\
+"#define KEY2JAVAHASH_NOT_NULL(x) (x)\n"\
+"#define KEY2INTHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( (x) ) )\n"\
+"#define KEY2LONGHASH(x) ( it.unimi.dsi.fastutil.HashCommon.mix( (long)( (x) ) ) )\n"\
+"#define INT(x) (x)\n"\
 "#endif\n"\
 "#endif\n"\
 \
 "#endif\n"\
 \
+"#ifndef KEY2JAVAHASH\n"\
+"#define KEY2JAVAHASH(x) KEY2JAVAHASH_NOT_NULL(x)\n"\
+"#endif\n"\
+\
 \
 \
 "/* Object/Reference-only definitions (values) */\n"\
