@@ -34,7 +34,7 @@ s */
 public class FastMultiByteArrayInputStream extends MeasurableInputStream implements RepositionableStream {
 
 	/** The number of bits of an array slice index. */
-	public final static int SLICE_BITS = 30;
+	public final static int SLICE_BITS = 10;
 
 	/** The maximum length of an array slice. */
 	public final static int SLICE_SIZE = 1 << SLICE_BITS;
@@ -42,18 +42,18 @@ public class FastMultiByteArrayInputStream extends MeasurableInputStream impleme
 	/** The mask to retrieve a slice offset. */
 	public final static int SLICE_MASK = SLICE_SIZE - 1;
 
-	/** The array of arrays backing the input stream. */
+	/** The array of arrays backing the input stream, plus an additional {@code null} entry. */
 	public byte[][] array;
 
+	/** The current array. */
+	public byte[] current;
+
 	/** The number of valid bytes in {@link #array}. */
 	public long length;
 
 	/** The current position. */
 	private long position;
 
-	/** The current mark, or -1 if no mark exists. */
-	private long mark;
-
 	/** Creates a new multi-array input stream loading it from a measurable input stream.
 	 *
 	 * @param is the input stream that will fill the array.
@@ -71,14 +71,16 @@ public class FastMultiByteArrayInputStream extends MeasurableInputStream impleme
 	
 	public FastMultiByteArrayInputStream( final InputStream is, long size ) throws IOException {
 		length = size;
-		array = new byte[ (int)( ( size + SLICE_SIZE - 1 ) / SLICE_SIZE ) ][];
+		array = new byte[ (int)( ( size + SLICE_SIZE - 1 ) / SLICE_SIZE ) + 1 ][];
 
-		for( int i = 0; i < array.length; i++ ) {
+		for( int i = 0; i < array.length - 1; i++ ) {
 			array[ i ] = new byte[ size >= SLICE_SIZE ? SLICE_SIZE : (int)size ];
 			// It is important *not* to use is.read() directly because of bug #6478546
 			if ( BinIO.loadBytes( is, array[ i ] ) != array[ i ].length ) throw new EOFException();
 			size -= array[ i ].length;
 		}
+
+		current = array[ 0 ];
 	}
 
 	/** Creates a new multi-array input stream sharing the backing arrays of another multi-array input stream. 
@@ -88,6 +90,7 @@ public class FastMultiByteArrayInputStream extends MeasurableInputStream impleme
 	public FastMultiByteArrayInputStream( final FastMultiByteArrayInputStream is ) {
 		this.array = is.array;
 		this.length = is.length;
+		this.current = array[ 0 ];
 	}
 
 
@@ -96,26 +99,13 @@ public class FastMultiByteArrayInputStream extends MeasurableInputStream impleme
 	 * @param array the backing array.
 	 */
 	public FastMultiByteArrayInputStream( final byte[] array ) {
-		this.array = new byte[ 1 ][];
-		this.array[ 0 ] = array;
-		this.length = array.length;
-	}
-
-
-
-	public boolean markSupported() {
-		return true;
-	}
-
-	public void reset() {
-		position = mark;
-	}
-
-	/** Closing a fast byte array input stream has no effect. */
-	public void close() {}
-
-	public void mark( final int dummy ) {
-		mark = position;
+		if ( array.length == 0 ) this.array = new byte[ 1 ][];
+		else {
+			this.array = new byte[ 2 ][];
+			this.array[ 0 ] = array;
+			this.length = array.length;
+			this.current = array;
+		}
 	}
 
 	/** Returns the number of bytes that can be read (or skipped over) from this input stream without blocking. 
@@ -127,54 +117,72 @@ public class FastMultiByteArrayInputStream extends MeasurableInputStream impleme
 	 */
 
 	public int available() {
-		if ( length - position > Integer.MAX_VALUE ) return Integer.MAX_VALUE;
-		return (int)( length - position );
+		return (int)Math.min( Integer.MAX_VALUE, length - position );
 	}
 
 	public long skip( long n ) {
-		if ( n <= length - position ) {
-			position += n;
-
-			return n;
-		}
-		n = length - position;
-		position = length;
+		if ( n > length - position ) n = length - position;
+		position += n;
+		updateCurrent();
 		return n;
 	}
 
 	public int read() {
 		if ( length == position ) return -1;
-		return array[ (int)( position >>> SLICE_BITS ) ][ (int)( position++ & SLICE_MASK ) ] & 0xFF;
+		final int disp = (int)( position++ & SLICE_MASK );
+		if ( disp == 0 ) updateCurrent();
+		return current[ disp ] & 0xFF;
 	}
 
 	public int read( final byte[] b, int offset, final int length ) {
-		if ( this.length == this.position ) return length == 0 ? 0 : -1;
-		int res, n = (int)Math.min( length, this.length - this.position ), m = n;
+		final long remaining = this.length - position;
+		if ( remaining == 0 ) return length == 0 ? 0 : -1;
+		int n = (int)Math.min( length, remaining );
+		final int m = n;
 
-		do {
-			res = Math.min( n, array[ (int)( position >>> SLICE_BITS ) ].length - (int)( position & SLICE_MASK ) );
-			System.arraycopy( array[ (int)( position >>> SLICE_BITS ) ], (int)( position & SLICE_MASK ),
-				b, offset, res );
+		for(;;) {
+			final int disp = (int)( position & SLICE_MASK );
+			if ( disp == 0 ) updateCurrent();
+			final int res = Math.min( n, current.length - disp );
+			System.arraycopy( current, disp, b, offset, res );
 				
 			n -= res;
-			offset += res;
 			position += res;
-			
-		} while( n > 0 );
-
-		return m;
+			if ( n == 0 ) return m;
+			offset += res;
+		}
 	}
 
+	private void updateCurrent() {
+		current = array[ (int)( position >>> SLICE_BITS ) ];
+	}
+	
 	public long position() {
 		return position;
 	}
 
 	public void position( final long newPosition ) {
 		position = Math.min( newPosition, length );
+		updateCurrent();
 	}
 
 	@Override
 	public long length() throws IOException {
 		return length;
 	}
+
+	/** NOP. */
+	public void close() {}
+
+	public boolean markSupported() {
+		return false;
+	}
+
+	public void mark( final int dummy ) {
+		throw new UnsupportedOperationException();
+	}
+
+	public void reset() {
+		throw new UnsupportedOperationException();
+	}
 }
