@@ -2,8 +2,11 @@ package it.unimi.dsi.fastutil.ints;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertTrue;
 import it.unimi.dsi.fastutil.Hash;
+import it.unimi.dsi.fastutil.HashCommon;
+import it.unimi.dsi.fastutil.ints.Int2IntMap.Entry;
 import it.unimi.dsi.fastutil.objects.ObjectIterator;
 
 import java.io.IOException;
@@ -15,6 +18,103 @@ import org.junit.Test;
 
 public class Int2IntLinkedOpenHashMapTest {
 
+
+	@Test
+	public void testWrapAround() {
+		Int2IntLinkedOpenHashMap m = new Int2IntLinkedOpenHashMap( 4, .5f );
+		assertEquals( 8, m.n );
+		// The following code inverts HashCommon.phiMix() and places strategically keys in slots 6, 7 and 0
+		m.put( HashCommon.invMix( 6 ), 0 );
+		m.put( HashCommon.invMix( 7 ), 0 );
+		m.put( HashCommon.invMix( 6 + 8 ), 0 );
+		assertNotEquals( 0, m.key[ 0 ] );
+		assertNotEquals( 0, m.key[ 6 ] );
+		assertNotEquals( 0, m.key[ 7 ] );
+		IntOpenHashSet keys = new IntOpenHashSet( m.keySet() );
+		IntIterator iterator = m.keySet().iterator();
+		IntOpenHashSet t = new IntOpenHashSet();
+		t.add( iterator.nextInt() );
+		t.add( iterator.nextInt() );
+		// Originally, this remove would move the entry in slot 0 in slot 6 and we would return the entry in 0 twice
+		iterator.remove(); 
+		t.add( iterator.nextInt() );
+		assertEquals( keys, t );
+	}	
+
+	@Test
+	public void testWrapAround2() {
+		Int2IntLinkedOpenHashMap m = new Int2IntLinkedOpenHashMap( 4, .75f );
+		assertEquals( 8, m.n );
+		// The following code inverts HashCommon.phiMix() and places strategically keys in slots 4, 5, 6, 7 and 0
+		m.put( HashCommon.invMix( 4 ), 0 );
+		m.put( HashCommon.invMix( 5 ), 0 );
+		m.put( HashCommon.invMix( 4 + 8 ), 0 );
+		m.put( HashCommon.invMix( 5 + 8 ), 0 );
+		m.put( HashCommon.invMix( 4 + 16 ), 0 );
+		assertNotEquals( 0, m.key[ 0 ] );
+		assertNotEquals( 0, m.key[ 4 ] );
+		assertNotEquals( 0, m.key[ 5 ] );
+		assertNotEquals( 0, m.key[ 6 ] );
+		assertNotEquals( 0, m.key[ 7 ] );
+		//System.err.println(Arraym.toString( m.key ));
+		IntOpenHashSet keys = new IntOpenHashSet( m.keySet() );
+		IntIterator iterator = m.keySet().iterator();
+		IntOpenHashSet t = new IntOpenHashSet();
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arraym.toString( m.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		//System.err.println(Arraym.toString( m.key ));
+		// Originally, this remove would move the entry in slot 0 in slot 6 and we would return the entry in 0 twice
+		assertTrue( t.add( iterator.nextInt() ) );
+		//System.err.println(Arraym.toString( m.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arraym.toString( m.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		assertEquals( 3, m.size() );
+		assertEquals( keys, t );
+	}	
+	
+	@Test
+	public void testWrapAround3() {
+		Int2IntLinkedOpenHashMap m = new Int2IntLinkedOpenHashMap( 4, .75f );
+		assertEquals( 8, m.n );
+		// The following code inverts HashCommon.phiMix() and places strategically keys in slots 5, 6, 7, 0 and 1
+		m.put( HashCommon.invMix( 5 ), 0 );
+		m.put( HashCommon.invMix( 5 + 8 ), 0 );
+		m.put( HashCommon.invMix( 5 + 16 ), 0 );
+		m.put( HashCommon.invMix( 5 + 32 ), 0 );
+		m.put( HashCommon.invMix( 5 + 64 ), 0 );
+		assertNotEquals( 0, m.key[ 5 ] );
+		assertNotEquals( 0, m.key[ 6 ] );
+		assertNotEquals( 0, m.key[ 7 ] );
+		assertNotEquals( 0, m.key[ 0 ] );
+		assertNotEquals( 0, m.key[ 1 ] );
+		//System.err.println(Arraym.toString( m.key ));
+		IntOpenHashSet keys = new IntOpenHashSet( m.keySet() );
+		IntIterator iterator = m.keySet().iterator();
+		IntOpenHashSet t = new IntOpenHashSet();
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arraym.toString( m.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arraym.toString( m.key ));
+		// Originally, this remove would move the entry in slot 0 in slot 6 and we would return the entry in 0 twice
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arraym.toString( m.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arraym.toString( m.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		assertEquals( 0, m.size() );
+		assertEquals( keys, t );
+	}	
+
+	
 	private static java.util.Random r = new java.util.Random( 0 );
 
 	private static int genKey() {
@@ -168,8 +268,8 @@ public class Int2IntLinkedOpenHashMapTest {
 		/* Now we play with iterators. */
 		{
 			java.util.ListIterator<?> i, j;
-			Object J;
-			Map.Entry<Integer,Integer> E, F;
+			Object J = null;
+			Map.Entry<Integer,Integer> E = null, F = null;
 			i = (java.util.ListIterator<?>)m.entrySet().iterator();
 			j = new java.util.LinkedList<Object>( t.entrySet() ).listIterator();
 			for ( int k = 0; k < 2 * n; k++ ) {
@@ -207,7 +307,7 @@ public class Int2IntLinkedOpenHashMapTest {
 		}
 		if ( t.size() > 0 ) {
 			java.util.ListIterator<Integer> i, j;
-			Object J;
+			Object J = null;
 			j = new java.util.LinkedList<Integer>( t.keySet() ).listIterator();
 			int e = r.nextInt( t.size() );
 			Object from;
@@ -364,6 +464,62 @@ public class Int2IntLinkedOpenHashMapTest {
 		for( int i = 50; i < 150; i++ ) assertEquals( i % 100, m.remove( i % 100 ) );
 	}
 
+	@Test
+	public void testRemove0() {
+		Int2IntLinkedOpenHashMap s = new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = -1; i <= 1; i++ ) assertEquals( 0, s.put( i, i ) );
+		s.remove( 0 );
+		IntIterator iterator = s.keySet().iterator();
+		IntOpenHashSet z = new IntOpenHashSet();
+		z.add( iterator.nextInt() );
+		z.add( iterator.nextInt() );
+		assertFalse( iterator.hasNext() );
+		assertEquals( new IntOpenHashSet( new int[] { -1, 1 } ), z );
+		
+		s = new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = -1; i <= 1; i++ ) assertEquals( 0, s.put( i, i ) );
+		iterator = s.keySet().iterator();
+		assertEquals( -1, iterator.nextInt() );
+		assertEquals( 0, iterator.nextInt() );
+		iterator.remove();
+		assertEquals( 1, iterator.nextInt() );
+		assertFalse( iterator.hasNext() );
+
+		assertFalse( s.containsKey( 0 ) );
+		
+		iterator = s.keySet().iterator();
+		assertEquals( -1, iterator.nextInt() );
+		assertEquals( 1, iterator.nextInt() );
+		assertFalse( iterator.hasNext() );
+	}
+
+	@Test
+	public void testFirtLast0() {
+		Int2IntLinkedOpenHashMap s;
+		
+		s = new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = 1; i < 100; i++ ) 
+			assertEquals( 0, s.put( i, i ) );
+		for( int i = 1; i < 100; i++ ) assertEquals( i, s.removeFirstInt() );
+		assertTrue( s.isEmpty() );
+		
+		s = new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = 0; i < 100; i++ ) assertEquals( 0, s.put( i, i ) );
+		for( int i = 100; i-- != 0; ) assertEquals( i, s.removeLastInt() );
+		assertTrue( s.isEmpty() );
+
+		s = new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = 100; i-- != 0; ) assertEquals( 0, s.put( i, i ) );
+		for( int i = 0; i < 100; i++ ) assertEquals( i, s.removeLastInt() );
+		assertTrue( s.isEmpty() );
+
+		s = new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = 100; i-- != 0; ) assertEquals( 0, s.put( i, i ) );
+		for( int i = 100; i-- != 0; ) assertEquals( i, s.removeFirstInt() );
+		assertTrue( s.isEmpty() );
+	}
+
+
 	@Test
 	public void testContainsValue() {
 		Int2IntLinkedOpenHashMap m = new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE );
@@ -492,4 +648,18 @@ public class Int2IntLinkedOpenHashMapTest {
 	public void testRemoveLastEmpty() {
 		new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE ).removeLastInt();
 	}
+
+	@Test
+	public void testFastIterator() {
+		Int2IntLinkedOpenHashMap s = new Int2IntLinkedOpenHashMap( Hash.DEFAULT_INITIAL_SIZE );
+		s.defaultReturnValue( -1 );
+		for( int i = 0; i < 100; i++ ) assertEquals( -1, s.put( i, i ) );
+		ObjectIterator<Entry> fastIterator = s.int2IntEntrySet().fastIterator();
+		Entry entry = fastIterator.next();
+		int key = entry.getIntKey();
+		entry.setValue( -1000 );
+		assertEquals( s.get( key ), -1000 );
+		fastIterator.remove();
+		assertEquals( s.get( key ), -1 );
+	}
 }
