@@ -1,12 +1,15 @@
 package it.unimi.dsi.fastutil.ints;
 
+import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
+import it.unimi.dsi.fastutil.Hash;
+import it.unimi.dsi.fastutil.HashCommon;
 
 import java.io.IOException;
-
-import it.unimi.dsi.fastutil.Hash;
+import java.util.Arrays;
 
 import org.junit.Ignore;
 import org.junit.Test;
@@ -50,10 +53,10 @@ public class IntOpenHashSetTest {
 		referenceInstance.retainAll(retainElements);
 
 		// print the correct result {586}
-		System.out.println("ref: " + referenceInstance);
+		// System.out.println("ref: " + referenceInstance);
 
 		// prints {586, 7379}, which is clearly wrong
-		System.out.println("ohm: " + instance);
+		// System.out.println("ohm: " + instance);
 
 		// Fails
 		assertEquals( referenceInstance, instance );
@@ -77,21 +80,153 @@ public class IntOpenHashSetTest {
 			}
 	}
 	
+	@Test
+	public void testRemove() {
+		IntOpenHashSet s = new IntOpenHashSet( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = 0; i < 100; i++ ) assertTrue( s.add( i ) );
+		for( int i = 0; i < 100; i++ ) assertFalse( s.remove( 100 + i ) );
+		for( int i = 50; i < 150; i++ ) assertTrue( Integer.toString( i % 100 ), s.remove( i % 100 ) );
+	}
+
+	@Test
+	public void testRemove0() {
+		IntOpenHashSet s = new IntOpenHashSet( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = -1; i <= 1; i++ ) assertTrue( s.add( i ) );
+		assertTrue( s.remove( 0 ) );
+		IntIterator iterator = s.iterator();
+		IntOpenHashSet z = new IntOpenHashSet();
+		z.add( iterator.nextInt() );
+		z.add( iterator.nextInt() );
+		assertFalse( iterator.hasNext() );
+		assertEquals( new IntOpenHashSet( new int[] { -1, 1 } ), z );
+		
+		s = new IntOpenHashSet( Hash.DEFAULT_INITIAL_SIZE );
+		for( int i = -1; i <= 1; i++ ) assertTrue( s.add( i ) );
+		iterator = s.iterator();
+		while( iterator.hasNext() ) if ( iterator.nextInt() == 0 ) iterator.remove();
+		
+		assertFalse( s.contains( 0 ) );
+		
+		iterator = s.iterator();
+		int[] content = new int[ 2 ];
+		content[ 0 ] = iterator.nextInt();
+		content[ 1 ] = iterator.nextInt();
+		assertFalse( iterator.hasNext() );
+		Arrays.sort( content );
+		assertArrayEquals( new int[] { -1, 1 }, content );
+	}
+	
+	@Test
+	public void testWrapAround() {
+		IntOpenHashSet s = new IntOpenHashSet( 4, .5f );
+		assertEquals( 8, s.n );
+		// The following code inverts HashCommon.phiMix() and places strategically keys in slots 6, 7 and 0
+		s.add( HashCommon.invMix( 6 ) );
+		s.add( HashCommon.invMix( 7 ) );
+		s.add( HashCommon.invMix( 6 + 8 ) );
+		assertNotEquals( 0, s.key[ 0 ] );
+		assertNotEquals( 0, s.key[ 6 ] );
+		assertNotEquals( 0, s.key[ 7 ] );
+		IntOpenHashSet keys = s.clone();
+		IntIterator iterator = s.iterator();
+		IntOpenHashSet t = new IntOpenHashSet();
+		t.add( iterator.nextInt() );
+		t.add( iterator.nextInt() );
+		// Originally, this remove would move the entry in slot 0 in slot 6 and we would return the entry in 0 twice
+		iterator.remove(); 
+		t.add( iterator.nextInt() );
+		assertEquals( keys, t );
+	}	
+
+	@Test
+	public void testWrapAround2() {
+		IntOpenHashSet s = new IntOpenHashSet( 4, .75f );
+		assertEquals( 8, s.n );
+		// The following code inverts HashCommon.phiMix() and places strategically keys in slots 4, 5, 6, 7 and 0
+		s.add( HashCommon.invMix( 4 ) );
+		s.add( HashCommon.invMix( 5 ) );
+		s.add( HashCommon.invMix( 4 + 8 ) );
+		s.add( HashCommon.invMix( 5 + 8 ) );
+		s.add( HashCommon.invMix( 4 + 16 ) );
+		assertNotEquals( 0, s.key[ 0 ] );
+		assertNotEquals( 0, s.key[ 4 ] );
+		assertNotEquals( 0, s.key[ 5 ] );
+		assertNotEquals( 0, s.key[ 6 ] );
+		assertNotEquals( 0, s.key[ 7 ] );
+		//System.err.println(Arrays.toString( s.key ));
+		IntOpenHashSet keys = s.clone();
+		IntIterator iterator = s.iterator();
+		IntOpenHashSet t = new IntOpenHashSet();
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arrays.toString( s.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		//System.err.println(Arrays.toString( s.key ));
+		// Originally, this remove would move the entry in slot 0 in slot 6 and we would return the entry in 0 twice
+		assertTrue( t.add( iterator.nextInt() ) );
+		//System.err.println(Arrays.toString( s.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arrays.toString( s.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		assertEquals( 3, s.size() );
+		assertEquals( keys, t );
+	}	
+	
+	@Test
+	public void testWrapAround3() {
+		IntOpenHashSet s = new IntOpenHashSet( 4, .75f );
+		assertEquals( 8, s.n );
+		// The following code inverts HashCommon.phiMix() and places strategically keys in slots 5, 6, 7, 0 and 1
+		s.add( HashCommon.invMix( 5 ) );
+		s.add( HashCommon.invMix( 5 + 8 ) );
+		s.add( HashCommon.invMix( 5 + 16 ) );
+		s.add( HashCommon.invMix( 5 + 32 ) );
+		s.add( HashCommon.invMix( 5 + 64 ) );
+		assertNotEquals( 0, s.key[ 5 ] );
+		assertNotEquals( 0, s.key[ 6 ] );
+		assertNotEquals( 0, s.key[ 7 ] );
+		assertNotEquals( 0, s.key[ 0 ] );
+		assertNotEquals( 0, s.key[ 1 ] );
+		//System.err.println(Arrays.toString( s.key ));
+		IntOpenHashSet keys = s.clone();
+		IntIterator iterator = s.iterator();
+		IntOpenHashSet t = new IntOpenHashSet();
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arrays.toString( s.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arrays.toString( s.key ));
+		// Originally, this remove would move the entry in slot 0 in slot 6 and we would return the entry in 0 twice
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arrays.toString( s.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		//System.err.println(Arrays.toString( s.key ));
+		assertTrue( t.add( iterator.nextInt() ) );
+		iterator.remove();
+		assertEquals( 0, s.size() );
+		assertEquals( keys, t );
+	}	
+	
 	@SuppressWarnings("boxing")
 	private static void checkTable( IntOpenHashSet s ) {
-		final boolean[] used = s.used;
-		final int[]key = s.key;
+		final int[] key = s.key;
 		assert ( s.n & -s.n ) == s.n : "Table length is not a power of two: " + s.n;
-		assert s.n == s.key.length;
-		assert s.n == used.length;
+		assert s.n == s.key.length - 1;
 		int n = s.n;
 		while ( n-- != 0 )
-			if ( used[ n ] && !s.contains( key[ n ] ) ) throw new AssertionError( "Hash table has key " + key[ n ]
+			if ( key[ n ] != 0 && !s.contains( key[ n ] ) ) throw new AssertionError( "Hash table has key " + key[ n ]
 					+ " marked as occupied, but the key does not belong to the table" );
+		
+		if ( s.containsNull && ! s.contains( 0 ) ) throw new AssertionError( "Hash table should contain zero by internal state, but it doesn't when queried" );
+		if ( ! s.containsNull && s.contains( 0 ) ) throw new AssertionError( "Hash table should not contain zero by internal state, but it does when queried" );
 
 		java.util.HashSet<Integer> t = new java.util.HashSet<Integer>();
 		for ( int i = s.size(); i-- != 0; )
-			if ( used[ i ] && !t.add( key[ i ] ) ) throw new AssertionError( "Key " + key[ i ] + " appears twice" );
+			if ( key[ i ] != 0 && !t.add( key[ i ] ) ) throw new AssertionError( "Key " + key[ i ] + " appears twice" );
 
 	}
 
@@ -99,9 +234,10 @@ public class IntOpenHashSetTest {
 		long totProbes = 0;
 		double totSquareProbes = 0;
 		int maxProbes = 0;
+		final int[] key = m.key;
 		final double f = (double)m.size / m.n;
 		for ( int i = 0, c = 0; i < m.n; i++ ) {
-			if ( m.used[ i ] ) c++;
+			if ( key[ i ] != 0 ) c++;
 			else {
 				if ( c != 0 ) {
 					final long p = ( c + 1 ) * ( c + 2 ) / 2;
